#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# pyDialer - "A simple and handy frontend for WvDial, fork of pyWvDial"
# Copyright (C) 2008 Giacomo Lozito
# Copyright (C) 2009 Aliaksandr Stelmachonak
# Licensed under GNU General Public License v2

import os.path
import pygtk
pygtk.require('2.0')
import gobject, gtk
import pyDialer.wvdial.config as wconfig
import pyDialer.guigtk.utils as gutils
from pyDialer.globs import *

def config_panel_show(parent_window, wcfg):
  if ( not ConfigInterface.window ):
    config_iface = ConfigInterface(parent_window, wcfg)
  else:
    ConfigInterface.window.present()

# NOTE: do not instantiate ConfigInterface directly from
# other modules; use function config_panel_show() to manage it
class ConfigInterface:

  # all of these variables are used as static variables,
  # and are never used as instance variables
  window = None

  def __init__(self, parent_window, wcfg):
    
    ConfigInterface.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    ConfigInterface.window.set_transient_for(parent_window)
    ConfigInterface.window.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
    ConfigInterface.window.set_title('pyDialer - config file')
    ConfigInterface.window.connect("destroy", self.destroy_cb)
    ConfigInterface.window.set_geometry_hints(None, 460, 260)
    ConfigInterface.window.set_border_width(4)

    self.icontheme = gtk.icon_theme_get_default()

    self.cfgfile_entry = gtk.Entry()
    self.cfgfile_entry.set_editable(False)
    self.cfgfile_entry.set_text(os.path.abspath(os.path.expanduser(wcfg)))
    # small trick to hide the cursor in this gtk.Entry (same color as base)
    estyle = self.cfgfile_entry.get_style()
    self.cfgfile_entry.modify_cursor(estyle.base[gtk.STATE_NORMAL], estyle.base[gtk.STATE_NORMAL])

    top_hbox1 = gtk.HBox(False, 4)
    top_hbox1.set_border_width(4)
    top_hbox1.pack_start(self.cfgfile_entry, True, True, 0)
    top_frame1 = gtk.Frame('Config File')
    top_frame1.add(top_hbox1)
    
    self.button_addsection = gtk.Button(stock=gtk.STOCK_ADD)
    self.button_addsection.connect("clicked", self.section_add_cb)
    self.button_remsection = gtk.Button(stock=gtk.STOCK_REMOVE)
    self.button_remsection.connect("clicked", self.section_remove_cb)
    top_hbbox2 = gtk.HButtonBox()
    top_hbbox2.add(self.button_addsection)
    top_hbbox2.add(self.button_remsection)
    top_hbox2 = gtk.HBox(False, 4)
    top_hbox2.set_border_width(4)
    top_hbox2.pack_start(top_hbbox2, True, True, 0)
    top_frame2 = gtk.Frame('Config File Sections')
    top_frame2.add(top_hbox2)
    
    top_hbox = gtk.HBox(False, 4)
    top_hbox.pack_start(top_frame1, True, True, 0)
    top_hbox.pack_start(top_frame2, False, True, 0)

    self.notebook = gtk.Notebook()
    pages_dict = dict()
    self.notebook.set_data("pages", pages_dict)

    self.invalid_label = gtk.Label('Invalid Config File')
    self.invalid_label.set_justify(gtk.JUSTIFY_CENTER)
    self.invalid_label.set_no_show_all(True)
    self.empty_label = gtk.Label('Empty Config File')
    self.empty_label.set_justify(gtk.JUSTIFY_CENTER)
    self.empty_label.set_no_show_all(True)

    bottom_hbbox = gtk.HButtonBox()
    bottom_hbbox.set_layout(gtk.BUTTONBOX_END)
    self.button_save = gtk.Button('_Save and Close')
    self.button_save.set_image(gtk.image_new_from_stock(gtk.STOCK_SAVE, gtk.ICON_SIZE_BUTTON))
    self.button_save.set_no_show_all(False)
    self.button_save.connect("clicked", self.save_and_close_cb)
    self.button_close = gtk.Button(stock=gtk.STOCK_CLOSE)
    self.button_close.connect_object("clicked", gtk.Widget.destroy, ConfigInterface.window)
    bottom_hbbox.add(self.button_save)
    bottom_hbbox.add(self.button_close)

    window_vbox = gtk.VBox(False, 4)
    window_vbox.pack_start(top_hbox, False, True, 0)
    window_vbox.pack_start(self.invalid_label, False, True, 0)
    window_vbox.pack_start(self.empty_label, False, True, 0)
    window_vbox.pack_start(self.notebook, True, True, 0)
    window_vbox.pack_start(gtk.HSeparator(), False, True, 0)    
    window_vbox.pack_start(bottom_hbbox, False, True, 0)
    ConfigInterface.window.add(window_vbox)

    self.load_config(wcfg)
    self.button_close.grab_focus()
    ConfigInterface.window.show_all()

  def load_config(self, wcfg):
    try:
      config = wconfig.WvDialConfig(wcfg)
      config.load_from_file()
    except (wconfig.WvDialConfig.ParsingError, wconfig.WvDialConfig.LoadFromFileError):
      config = None
      self.set_invalid_config(True, 'Invalid Config File')
      self.set_empty_config(False)
    except wconfig.WvDialConfig.ConflictingOptionsError, (section, opt1, opt2):
      config = None
      self.set_invalid_config(True, 'Conflicting options in section \'%s\':\n\n\'%s\' and \'%s\'\n\nplease remove one of them.' % (section, opt1, opt2))
      self.set_empty_config(False)
    else:
      self.set_invalid_config(False)
      config_sections = config.get_sections()
      self.notebook.set_data("cfg", config)
      if ( len(config_sections) < 1 ):
        self.set_empty_config(True)
      else:
        self.set_empty_config(False)
        for section in config_sections:
          self.section_add(section)
          self.section_set_values(section, config)

  def set_invalid_config(self, isInvalid, invalid_message=None):
    if (isInvalid == True):
      self.button_addsection.set_sensitive(False)
      self.button_remsection.set_sensitive(False)
      self.button_save.set_no_show_all(True)
      self.invalid_label.set_no_show_all(False)
      if (invalid_message):
        self.invalid_label.set_text(invalid_message)
      self.button_save.hide()
      self.invalid_label.show()
    else:
      self.button_addsection.set_sensitive(True)
      self.button_remsection.set_sensitive(True)
      self.button_save.set_no_show_all(False)
      self.invalid_label.set_no_show_all(True)
      self.button_save.show()
      self.invalid_label.hide()

  def set_empty_config(self, isEmpty):
    if (isEmpty == True):
      self.empty_label.set_no_show_all(False)
      self.empty_label.show()
    else:
      self.empty_label.set_no_show_all(True)
      self.empty_label.hide()

  def section_add_cb(self, button):
    dialog = gtk.Dialog('Add Section', ConfigInterface.window, gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
    dialog.set_default_response(gtk.RESPONSE_ACCEPT)
    label = gtk.Label('Please enter section name (single word):')
    entry = gtk.Entry()
    entry.set_activates_default(True)
    vbox = gtk.VBox(False, 2)
    vbox.set_border_width(4)
    vbox.pack_start(label, True, True, 0)
    vbox.pack_start(entry, True, True, 0)
    frame = gtk.Frame()
    frame.set_border_width(4)
    frame.add(vbox)
    dialog.vbox.pack_start(frame, True, True, 0)
    dialog.show_all()
    response = dialog.run()
    if ( response == gtk.RESPONSE_ACCEPT ):
      section_name = entry.get_text()
    else:
      section_name = ''
    dialog.destroy()
    # check section_name validity
    section_name = (section_name.expandtabs(1)).replace(' ','')
    if ( section_name != '' ):
      # the section name must not have been used already
      pages_dict = self.notebook.get_data("pages")
      if ( section_name in pages_dict ):
        msg_dlg = gtk.MessageDialog(parent=ConfigInterface.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                    type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
                                    message_format='A section with name \'%s\' already exists.' % section_name)
        msg_dlg.run()
        msg_dlg.destroy()
      else:
        self.section_add(section_name)

  def section_remove_cb(self, button):
    current_page = self.notebook.get_current_page()
    if ( current_page != -1 ):
      section_name = (self.notebook.get_nth_page(current_page)).get_data('name')
      yn_dlg = gtk.MessageDialog(parent=ConfigInterface.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                 type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO,
                                 message_format='Delete section \'%s\'?' % section_name)
      yn_dlg.set_default_response(gtk.RESPONSE_NO)
      response = yn_dlg.run()
      yn_dlg.destroy()
      if ( response == gtk.RESPONSE_YES ):
        self.section_remove(section_name)

  def section_add(self, section_name):
    section_nb = gtk.Notebook()
    section_nb.set_tab_pos(gtk.POS_LEFT)
    section_nb.set_data('name', section_name)
    # create two dictionaries (gtk.Entry widgets and gtk.RadioButton widgets)
    text_options = dict()
    choice_options = dict()
    section_nb.set_data('text_options', text_options)
    section_nb.set_data('choice_options', choice_options)
    # get minimum label width from the longest label
    elabel = gtk.Label('Check def.route:')
    elayout = elabel.get_layout()
    mlw = elayout.get_pixel_size()[0]
    elabel.destroy()
    
    if ( self.icontheme.has_icon('document') ):
      label_obj = gutils.ImgLabelObj(expandLabel=True, expandIcon=False, putIconFirst=True, isLabelSelectable=False, isLabelEllipsizable=False, xAlignLabel=0.5, yAlignLabel=0.5, useEvbox=False, evBoxPadding=0)
      label_obj.update_label(section_name, False)
      label_obj.update_icon(pixbuf=self.icontheme.load_icon('document', gtk.icon_size_lookup(gtk.ICON_SIZE_MENU)[1], 0))
      label = label_obj.get_widget()
      label.show_all()
    else:
      label = gtk.Label(section_name)

    # if this is the first section, remove the empty config message
    if ( self.notebook.get_n_pages() < 1 ):
      self.set_empty_config(False)
    if ( section_name == 'Defaults' ):
      self.notebook.prepend_page(section_nb, label)
    else:
      self.notebook.append_page(section_nb, label)
    pages_dict = self.notebook.get_data("pages")
    pages_dict[section_name] = section_nb

    def separator(text):
      sep = gtk.HBox(False, 2)
      sep.pack_start(gtk.HSeparator(), True, True, 0)
      sep.pack_start(gtk.Label(text), False, True, 0)
      sep.pack_start(gtk.HSeparator(), True, True, 0)
      return sep

    def text_entry(option_name, label_text, label_len=0):
      label = gtk.Label(label_text)
      label.set_alignment(0, 0.5)
      label.set_size_request(label_len, -1)
      entry = gtk.Entry()
      hbox = gtk.HBox(False, 2)
      hbox.pack_start(label, False, True, 0)
      hbox.pack_start(entry, True, True, 0)
      text_options[option_name] = entry
      return (hbox, label, entry)

    def digit_entry(option_name, label_text, label_len=0):
      # a text_entry that accepts digits only
      def instext_cb(entry, new_text, ntl, pos):
        if ( not new_text.isdigit() ):
          entry.stop_emission("insert-text")
      le = text_entry(option_name, label_text, label_len)
      le[2].connect("insert-text", instext_cb)
      return le

    def choice_entry(option_name, label_text, label_len=0):
      label = gtk.Label(label_text)
      label.set_alignment(0, 0.5)
      label.set_size_request(label_len, -1)
      ch_def = gtk.RadioButton(None, 'Def')
      ch_on = gtk.RadioButton(ch_def, 'On')
      ch_off = gtk.RadioButton(ch_def, 'Off')
      hbox = gtk.HBox(False, 2)
      hbox.pack_start(label, True, True, 0)
      hbox.pack_start(ch_def, False, True, 0)
      hbox.pack_start(ch_on, False, True, 0)
      hbox.pack_start(ch_off, False, True, 0)
      choice_options[option_name] = {'def':ch_def, 'on':ch_on, 'off':ch_off}
      ch_def.set_active(True) # default
      return hbox
    
    modem_table = gtk.Table(11, 2, True)
    modem_table.set_border_width(4)
    modem_table.set_row_spacings(2)
    modem_table.set_col_spacings(16)
    section_nb.append_page(modem_table, gtk.Label('Modem'))
    modem_sep = separator('Modem')
    modem_table.attach(modem_sep, 0, 2, 0, 1, gtk.EXPAND|gtk.FILL, gtk.FILL)
    modem_table.set_row_spacing(0, 8)

    modem_dev_le = text_entry('Modem', 'Modem:', mlw)
    modem_baud_le = digit_entry('Baud', 'Baud:', mlw)
    #modem_baud_label = gtk.Label('Baud:')
    #modem_baud_label.set_alignment(0, 0.5)
    #modem_baud_entry = gtk.combo_box_entry_new_text()
    #modem_baud_entry.append_text('56700')
    #modem_baud_entry.append_text('115200')
    #modem_baud_entry.append_text('230400')
    #modem_baud_entry.append_text('460800')
    #modem_baud_ali = gtk.Alignment(1, 1, 1, 1)
    #modem_baud_ali.set_padding(0, 0, 0)
    #modem_baud_ali.add(modem_baud_entry)
    modem_table.attach(modem_dev_le[0], 0, 1, 1, 2, gtk.EXPAND|gtk.FILL, gtk.FILL)
    modem_table.attach(modem_baud_le[0], 0, 1, 2, 3, gtk.EXPAND|gtk.FILL, gtk.FILL)

    modem_init_table = gtk.Table(9, 2, False)
    modem_init_table.set_row_spacings(0)
    modem_init_table.set_col_spacings(4)
    for i in range(1,10):
      modem_init_name = 'Init%i' % i
      modem_init_label = gtk.Label('%i:' % i)
      modem_init_label.set_alignment(0, 0.5)
      modem_init_entry = gtk.Entry()
      modem_init_table.attach(modem_init_label, 0, 1, (i-1), i, gtk.FILL, gtk.FILL)
      modem_init_table.attach(modem_init_entry, 1, 2, (i-1), i, gtk.EXPAND|gtk.FILL, gtk.FILL)
      text_options[modem_init_name] = modem_init_entry
    modem_init_ali = gtk.Alignment(1, 1, 1, 1)
    modem_init_ali.set_padding(0, 4, 4, 4)
    modem_init_ali.add(modem_init_table)
    modem_init_frame = gtk.Frame()
    modem_init_frame_l = gtk.Label()
    modem_init_frame_l.set_markup('<small>Modem Init Strings:</small>')
    modem_init_frame_l.set_padding(0, 2)
    modem_init_frame.set_label_widget(modem_init_frame_l)
    modem_init_frame.set_label_align(0.5, 0)
    modem_init_frame.add(modem_init_ali)
    modem_init_vbox = gtk.VBox(False, 2)
    modem_init_vbox.pack_start(modem_init_frame, False, True, 0)
    modem_table.attach(modem_init_vbox, 1, 2, 1, 11, gtk.EXPAND|gtk.FILL, gtk.FILL)

    dial_table = gtk.Table(9, 2, True)
    dial_table.set_border_width(4)
    dial_table.set_row_spacings(2)
    dial_table.set_col_spacings(16)
    section_nb.append_page(dial_table, gtk.Label('Dialing'))
    dial_sep = separator('Dialing')
    dial_table.attach(dial_sep, 0, 2, 0, 1, gtk.EXPAND|gtk.FILL, gtk.FILL)
    dial_table.set_row_spacing(0, 8)

    dial_phone_le = text_entry('Phone', 'Phone:', mlw)
    dial_table.attach(dial_phone_le[0], 0, 1, 1, 2, gtk.EXPAND|gtk.FILL, gtk.FILL)
    dial_prefix_le = text_entry('Dial Prefix', 'Prefix:', mlw)
    dial_table.attach(dial_prefix_le[0], 0, 1, 2, 3, gtk.EXPAND|gtk.FILL, gtk.FILL)
    dial_prefix_le = text_entry('Dial Command', 'Command:', mlw)
    dial_table.attach(dial_prefix_le[0], 0, 1, 3, 4, gtk.EXPAND|gtk.FILL, gtk.FILL)
    dial_attemps_nle = digit_entry('Dial Attempts', 'Attemps:', mlw)
    dial_table.attach(dial_attemps_nle[0], 0, 1, 4, 5, gtk.EXPAND|gtk.FILL, gtk.FILL)
    dial_idle_nle = digit_entry('Idle Seconds', 'Idle secs:', mlw)
    dial_table.attach(dial_idle_nle[0], 0, 1, 5, 6, gtk.EXPAND|gtk.FILL, gtk.FILL)

    dial_phones_table = gtk.Table(4, 2, False)
    dial_phones_table.set_row_spacings(0)
    dial_phones_table.set_col_spacings(4)
    for i in range(1,5):
      dial_phones_name = 'Phone%i' % i
      dial_phones_label = gtk.Label('%i:' % i)
      dial_phones_label.set_alignment(0, 0.5)
      dial_phones_entry = gtk.Entry()
      dial_phones_table.attach(dial_phones_label, 0, 1, (i-1), i, gtk.FILL, gtk.FILL)
      dial_phones_table.attach(dial_phones_entry, 1, 2, (i-1), i, gtk.EXPAND|gtk.FILL, gtk.FILL)
      text_options[dial_phones_name] = dial_phones_entry
    dial_phones_ali = gtk.Alignment(1, 1, 1, 1)
    dial_phones_ali.set_padding(0, 4, 4, 4)
    dial_phones_ali.add(dial_phones_table)
    dial_phones_frame = gtk.Frame()
    dial_phones_frame_l = gtk.Label()
    dial_phones_frame_l.set_markup('<small>Other Phone Numbers:</small>')
    dial_phones_frame_l.set_padding(0, 2)
    dial_phones_frame.set_label_widget(dial_phones_frame_l)
    dial_phones_frame.set_label_align(0.5, 0)
    dial_phones_frame.add(dial_phones_ali)
    dial_phoes_vbox = gtk.VBox(False, 2)
    dial_phoes_vbox.pack_start(dial_phones_frame, False, True, 0)
    dial_table.attach(dial_phoes_vbox, 1, 2, 1, 7, gtk.EXPAND|gtk.FILL, gtk.FILL)

    dial_abortonbusy_ch = choice_entry('Abort on Busy', 'Abort on busy:', mlw)
    dial_table.attach(dial_abortonbusy_ch, 0, 1, 7, 8, gtk.FILL, gtk.FILL)
    dial_abortonnodial_ch = choice_entry('Abort on No Dialtone', 'Abort on no dialtone:', mlw)
    dial_table.attach(dial_abortonnodial_ch, 1, 2, 7, 8, gtk.FILL, gtk.FILL)
    dial_abortonbusy_ch = choice_entry('Auto Reconnect', 'Auto reconnect:', mlw)
    dial_table.attach(dial_abortonbusy_ch, 0, 1, 8, 9, gtk.FILL, gtk.FILL)
    dial_abortonbusy_ch = choice_entry('Carrier Check', 'Carrier check:', mlw)
    dial_table.attach(dial_abortonbusy_ch, 1, 2, 8, 9, gtk.FILL, gtk.FILL)

    isp_table = gtk.Table(8, 2, True)
    isp_table.set_border_width(4)
    isp_table.set_row_spacings(2)
    isp_table.set_col_spacings(16)
    section_nb.append_page(isp_table, gtk.Label('ISP'))
    isp_sep = separator('Internet Service Provider')
    isp_table.attach(isp_sep, 0, 2, 0, 1, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_table.set_row_spacing(0, 8)

    isp_login_le = text_entry('Login', 'Login:', mlw)
    isp_table.attach(isp_login_le[0], 0, 1, 1, 2, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_pass_le = text_entry('Password', 'Password:', mlw)
    isp_table.attach(isp_pass_le[0], 1, 2, 1, 2, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_lprompt_le = text_entry('Login Prompt', 'Login prompt:', mlw)
    isp_table.attach(isp_lprompt_le[0], 0, 1, 2, 3, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_pprompt_le = text_entry('Password Prompt', 'Pass. prompt:', mlw)
    isp_table.attach(isp_pprompt_le[0], 1, 2, 2, 3, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_defreply_le = text_entry('Default Reply', 'Default reply:', mlw)
    isp_table.attach(isp_defreply_le[0], 0, 1, 3, 4, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_lprompt_ch = choice_entry('Ask Password', 'Ask password:', mlw)
    isp_table.attach(isp_lprompt_ch, 1, 2, 3, 4, gtk.FILL, gtk.FILL)
    isp_forceaddr_le = text_entry('Force Address', 'Force address:', mlw)
    isp_table.attach(isp_forceaddr_le[0], 0, 1, 4, 5, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_remotename_le = text_entry('Remote Name', 'Remote name:', mlw)
    isp_table.attach(isp_remotename_le[0], 1, 2, 4, 5, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_dnsauto_ch = choice_entry('Auto DNS', 'Auto DNS:', mlw)
    isp_table.attach(isp_dnsauto_ch, 0, 1, 5, 6, gtk.FILL, gtk.FILL)
    isp_dnscheck_ch = choice_entry('Check DNS', 'Check DNS:', mlw)
    isp_table.attach(isp_dnscheck_ch, 1, 2, 5, 6, gtk.FILL, gtk.FILL)
    isp_dnstest1_le = text_entry('DNS Test1', 'Dns Test #1:', mlw)
    isp_table.attach(isp_dnstest1_le[0], 0, 1, 6, 7, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_dnstest2_le = text_entry('DNS Test2', 'Dns Test #2:', mlw)
    isp_table.attach(isp_dnstest2_le[0], 1, 2, 6, 7, gtk.EXPAND|gtk.FILL, gtk.FILL)
    isp_defroute_ch = choice_entry('Check Def Route', 'Check def.route:', mlw)
    isp_table.attach(isp_defroute_ch, 0, 1, 7, 8, gtk.FILL, gtk.FILL)
    isp_stupidmode_ch = choice_entry('Stupid Mode', 'Stupid mode:', mlw)
    isp_table.attach(isp_stupidmode_ch, 1, 2, 7, 8, gtk.FILL, gtk.FILL)

    system_table = gtk.Table(2, 2, True)
    system_table.set_border_width(4)
    system_table.set_row_spacings(2)
    system_table.set_col_spacings(16)
    section_nb.append_page(system_table, gtk.Label('System'))
    system_sep = separator('System')
    system_table.attach(system_sep, 0, 2, 0, 1, gtk.EXPAND|gtk.FILL, gtk.FILL)
    system_table.set_row_spacing(0, 8)

    system_pppdpath_le = text_entry('PPPD Path', 'PPPD path:', mlw)
    system_table.attach(system_pppdpath_le[0], 0, 1, 1, 2, gtk.EXPAND|gtk.FILL, gtk.FILL)
    system_pppdnew_ch = choice_entry('New PPPD', 'New PPPD:', mlw)
    system_table.attach(system_pppdnew_ch, 1, 2, 1, 2, gtk.FILL, gtk.FILL)
    section_nb.show_all()

  def section_remove(self, section_name):
    pages_dict = self.notebook.get_data("pages")
    page = pages_dict[section_name]
    current_page = self.notebook.page_num(page)
    self.notebook.remove_page(current_page)
    page.destroy()
    del pages_dict[section_name]
    # if this was the last section, put the empty config message
    if ( self.notebook.get_n_pages() < 1 ):
      self.set_empty_config(True)

  def section_set_values(self, section_name, config):
    # get options dictionaries for the given section
    pages_dict = self.notebook.get_data('pages')
    section_nb = pages_dict[section_name]
    text_options = section_nb.get_data('text_options')
    choice_options = section_nb.get_data('choice_options')
    # iterate through text_options dictionary and set config-provided values
    for option_name, option_entry in text_options.iteritems():
      value = config.get_value(section_name, option_name)
      if ( value ):
        option_entry.set_text(value)
    # iterate through choice_options dictionary and set config-provided values
    for option_name, option_radiodict in choice_options.iteritems():
      value = config.get_value(section_name, option_name)
      if ( value ):
        if ( value in ('on', 'yes', '1') ):
          option_radiodict['on'].set_active(True)
        elif ( value in ('off', 'no', '0') ):
          option_radiodict['off'].set_active(True)
        else:
          option_radiodict['def'].set_active(True)

  def save_and_close_cb(self, button):
    yn_dlg = gtk.MessageDialog(parent=ConfigInterface.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                               type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO,
                               message_format='Config file will be overwritten.\nSave changes?')
    yn_dlg.set_default_response(gtk.RESPONSE_NO)
    response = yn_dlg.run()
    yn_dlg.destroy()
    if ( response == gtk.RESPONSE_YES ):
      # get config
      config = self.notebook.get_data("cfg")
      # get pages
      pages_dict = self.notebook.get_data('pages')
      # pick a list of sections in config (to be compared with pages_dict)
      sections_from_config = config.get_sections()
      # for each page (section), pick values and commit in config
      for section_name, section_page in pages_dict.iteritems():
        # create section, just in case it doesn't exist already
        config.add_section(section_name)
        text_options = section_page.get_data('text_options')
        choice_options = section_page.get_data('choice_options')
        # commit values from text_options dictionary in config
        for option_name, option_entry in text_options.iteritems():
          value = option_entry.get_text()
          if ( value != '' ):
            config.set_value(section_name, option_name, value)
          else:
            config.unset_value(section_name, option_name)
        # commit values from choice_options dictionary in config
        for option_name, option_radiodict in choice_options.iteritems():
          if ( option_radiodict['on'].get_active() ):
            config.set_value(section_name, option_name, 'on')
          elif ( option_radiodict['off'].get_active() ):
            config.set_value(section_name, option_name, 'off')
          else:
            config.unset_value(section_name, option_name)
        # if this section was in config already, remove it from the list
        if ( section_name in sections_from_config ):
          sections_from_config.remove(section_name)
      # after iteration, sections_from_config only contains those section that
      # were not found while iterating (alas, those that have been removed with the
      # 'remove section' button and should be removed from the new config as well)
      for section_name in sections_from_config:
        config.remove_section(section_name)
      # save the new config to file
      try:
        config.save_to_file()
      except wconfig.WvDialConfig.SaveToFileError:
        msg_dlg = gtk.MessageDialog(parent=ConfigInterface.window, flags=gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                    type=gtk.MESSAGE_ERROR, buttons=gtk.BUTTONS_CLOSE,
                                    message_format='Unable to save in config file.\nPlease check file permissions.')
        msg_dlg.run()
        msg_dlg.destroy()
      else:
        # done, close the window
        ConfigInterface.window.destroy()

  def destroy_cb(self, window):
    ConfigInterface.window = None
